import React, { PureComponent } from 'react';
import PropTypes from 'prop-types';
import { message, Upload, Icon, Modal } from 'antd';
import styles from './index.less';
import { getSignature } from '../../services/resource';

const getFileList = (files) => {
  if (Array.isArray(files)) {
    return files.map((item, key) => {
      const urlArr = item.full_url.split('/');
      return { url: item.full_url, id: '1', uid: key, name: urlArr[urlArr.length - 1], status: 'done' };
    });
  }
  if (files && !!files.length) {
    const filesArr = files.split('/');
    return [{ uid: -1, url: files, name: filesArr[filesArr.length - 1], status: 'done' }];
  }
  return '';
}

const renderAccept = (accept) => {
  if (!accept) {
    return null;
  }
  if (['image', 'video', 'audio'].find(ext => ext === accept)) {
    return `${accept}/*`;
  }
  if (accept === 'zip') {
    return 'application/zip,application/x-zip,application/x-zip-compressed';
  }
  return `.${accept}`;
}

export default class Uploader extends PureComponent {
  static propTypes = {
    files: PropTypes.oneOfType([PropTypes.array, PropTypes.string]),
    onUpload: PropTypes.func.isRequired,
    multiple: PropTypes.oneOfType([PropTypes.bool, PropTypes.number]),
    disabled: PropTypes.bool,
    path: PropTypes.string,
    accept: PropTypes.string,
  };

  constructor(props) {
    super(props);
    this.state = {
      previewVisible: false,
      previewImage: '',
      files: getFileList(props.files),
      signature: {},
      fileName: '',
    };
  }

  componentWillReceiveProps(nextProps) {
    // // 当前属性中的文件对象是一个列表并且接受到的属性中files不为空
    // if (Array.isArray(this.props.files) && !this.props.files.length && !!nextProps.files.length) {
    //   this.setState({ files: getFileList(nextProps.files) });
    // }
      this.setState({ files: getFileList(nextProps.files) });
  }

  render() {
    const { previewVisible, previewImage, files, signature, fileName } = this.state;
    const { multiple = 1, onUpload, disabled, path, accept } = this.props;

    // 根据单图/多图上传模式来选择返回一个还是多个文件对象
    const renderFiles = (fileList) => {
      const newFiles = fileList.map(file => {
        return file.response ? file.response.data.file : file;
      });
      if (multiple === 1) {
        return newFiles[0];
      }
      return newFiles;
    }

    const uploadProps = {
      accept: renderAccept(accept),
      action: signature.host,
      headers: {
        'Authorization': `OSS${signature.accessid}:${signature.signature}`,
      },
      data: {
        policy: signature.policy,
        signature: signature.signature,
        OSSAccessKeyId: signature.accessid,
        key: `${signature.dir}${fileName}`,
        success_action_status: '200',
      },
      disabled,
      listType: 'picture-card',
      fileList: files,
      multiple: multiple === true,
      onPreview: (file) => {
        this.setState({ previewImage: file.url || file.thumbUrl, previewVisible: true });
      },
      beforeUpload: (file, fileList) => {
        // 根据图片名称，转换成路径地址
        this.setState({
          fileName: file.name.replace(/-/g, '/')
        });
        // 进行签名校验，失效则刷新签名
        return getSignature({ fileName: file.name }).then(res => this.setState({signature: { ...res.data }}));
      },
      onChange: ({ file, fileList }) => {
        // 检查图片大小，不能超过5M
        if (file && file.size > 5 * 1024 * 1024) {
          message.error('图片大小不能超过5M!');
          return false;
        }
        // 满足要求，更新files对象
        this.setState({ files: fileList });
        // 检查上传过程中图片的状态，上传成功回传fileName, signature, file
        if (file.percent === 100 && file.status === 'done') {
          onUpload(renderFiles(fileList, 1), fileName, signature);
        } else if (file.status === 'error') {
          message.error('图片上传失败!');
          // this.setState({ files: [] });
        }
      },
      onRemove: (file) => {
        if (disabled) {
          return false;
        }
        const fileList = this.state.files.filter(item => item.uid !== file.uid);
        onUpload(renderFiles(fileList, 0), fileName, signature);
        return true;
      },
    };

    const modalProps = {
      visible: previewVisible,
      footer: null,
      onCancel: () => this.setState({ previewVisible: false }),
    };

    const uploadButton = (
      <div>
        <Icon type="plus" />
        <div className="ant-upload-text">点击上传</div>
      </div>
    );

    return (
      <div>
        <Upload {...uploadProps}>
          {multiple === true ? uploadButton : (files.length < multiple && uploadButton)}
        </Upload>
        <Modal {...modalProps}>
          <img className={styles.previewImage} alt="" src={previewImage} />
        </Modal>
      </div>
    );
  }
}
