import React, { Component } from 'react';
import { connect } from 'dva';
import classnames from 'classnames';
import {
  Radio,
  Row,
  Col,
  Table,
  Form,
  Card,
  Select,
  Input,
  Icon,
  Button,
  DatePicker,
} from 'antd';
import moment from 'moment';
import AnimTableBody from '../../components/Animation/AnimTableBody';
import { productType, orderStatuses } from '../../utils/config';
import styles from './index.less';

const { RangePicker } = DatePicker;

@Form.create()
@connect(state => ({ snapshot: state.snapshot }))
export default class SoldProductList extends Component {
  getFormValue = () => {
    const { form } = this.props;
    const { getFieldsValue } = form;
    const filters = getFieldsValue();
    if (filters && filters.time) {
      filters.timeBegin = filters.time[0].format('X');
      filters.timeEnd = filters.time[1].format('X');
      delete filters.time;
    }
    if (filters.orderStatus == 'all') {
      delete filters.orderStatus;
    }
    return filters;
  }

  handleFilterSubmit = (e) => {
    e.preventDefault();
    const filters = this.getFormValue();
    this.props.dispatch({ type: 'snapshot/query', payload: filters });
  }

  handleFilterReset = () => {
    this.props.form.resetFields();
    this.props.dispatch({ type: 'snapshot/query', payload: {} });
  }

  handleTableChange = (pagination) => {
    const filters = { ...this.getFormValue(), ...pagination };
    this.props.dispatch({ type: 'snapshot/query', payload: filters });
  }

  render() {
    const { snapshot, form } = this.props;
    const { getFieldDecorator } = form;
    const { list, listLoading, pagination } = snapshot;

    const columns = [{
      title: '产品编号',
      dataIndex: 'productCode',
      key: 1,
    },{
      title: '产品名称',
      dataIndex: 'productName',
      key: 2,
    },{
      title: '类型',
      dataIndex: 'productType',
      key: 3,
      render: (text, record) => productType[text],
    },{
      title: '终端编号',
      dataIndex: 'userCode',
      key: 4,
    },{
      title: '终端名称',
      dataIndex: 'userName',
      key: 5,
    },{
      title: '校区编号',
      dataIndex: 'campusCode',
      key: 6,
    },{
      title: '校区名称',
      dataIndex: 'campusName',
      key: 7,
    },{
      title: '渠道名称',
      dataIndex: 'merchantName',
      key: 8,
    },{
      title: '供应商价格(¥)',
      dataIndex: 'cpPrice',
      key: 9,
    },{
      title: '渠道价格(¥)',
      dataIndex: 'merchantPrice',
      key: 10,
    },{
      title: '终端价格(¥)',
      dataIndex: 'terminalPrice',
      key: 11,
    },{
      title: '数量',
      dataIndex: 'quantity',
      key: 12,
    },{
      title: '单位',
      dataIndex: 'chargeUnit',
      key: 13,
    },{
      title: '创建时间',
      dataIndex: 'gmtCreated',
      key: 14,
      render: (text, record) => moment(text).format('YYYY-MM-DD HH:mm:ss'),
    }];

    const newPagination = !!pagination && { ...pagination, showSizeChanger: true, showQuickJumper: true, showTotal: total => `共 ${total} 条`};
    const AnimationTableBody = (props) => (<AnimTableBody {...props}/>);

    return (
      <Card>
        <Form layout="inline" className={styles.tableSearchForm} onSubmit={this.handleFilterSubmit}>
          <Row gutter={24}>
            <Col span={12}>
              <Form.Item label="起始时间">
                {getFieldDecorator('time', {
                })(<RangePicker />)}
              </Form.Item>
            </Col>
            <Col span={12}>
              <Form.Item label="终端编号">
                {getFieldDecorator('userCode', {
                })(<Input placeholder="请输入" />)}
              </Form.Item>
            </Col>
          </Row>
          <Row gutter={24}>
            <Col span={12}>
              <Form.Item label="校区编号">
                {getFieldDecorator('campusCode', {
                })(<Input placeholder="请输入" />)}
              </Form.Item>
            </Col>
            <Col span={12}>
              <Form.Item label="校区名称">
                {getFieldDecorator('campusName', {
                })(<Input placeholder="请输入" />)}
              </Form.Item>
            </Col>
          </Row>
          <Row>
            <Col span={24} style={{ textAlign: 'left' }}>
              <Form.Item label="订单状态">
                {getFieldDecorator('orderStatus', {
                  initialValue: 'all',
                })(
                  <Radio.Group style={{ width: '100%' }}>
                    <Radio key="all" value="all">不限</Radio>
                    {Object.keys(orderStatuses).map(item => <Radio key={item} value={item}>{orderStatuses[item]}</Radio>)}
                  </Radio.Group>
                )}
              </Form.Item>
            </Col>
          </Row>
          <Row>
            <Col span={24} style={{ textAlign: 'right' }}>
              <Button type="primary" htmlType="submit">搜索</Button>
              <Button onClick={this.handleFilterReset} style={{ marginLeft: 8 }}>重置</Button>
            </Col>
          </Row>
        </Form>
        <Table
          bordered
          loading={listLoading}
          onChange={this.handleTableChange}
          rowKey={(record) => record.id}
          columns={columns}
          dataSource={list}
          pagination={newPagination}
          scroll={{ x: 1800 }}
          className={classnames({ [styles.table]: true, [styles.motion]: true })}
          components={{
            body: { wrapper: AnimationTableBody }
          }}
        />
      </Card>
    );
  }
}
