import React, { PureComponent } from 'react';
import { routerRedux } from 'dva/router';
import PropTypes from 'prop-types';
import queryString from 'query-string';
import { connect } from 'dva';
import { Spin, Badge, Table, Radio, Card, Form, Input, Icon, Button, Select } from 'antd';
import PageHeaderLayout from '../../../layouts/PageHeaderLayout';
import ItemSortModal from './modal';
import { Codes, itemStatuses, tagType } from '../../../utils/config';

@Form.create()
@connect(state => ({
  tagDetail: state.tagDetail,
  group: state.group,
}))
export default class TagDetail extends PureComponent {
  static propTypes = {
    tagDetail: PropTypes.object,
  };

  componentDidMount() {
    const { dispatch } = this.props;
    dispatch({
      type: 'group/query',
      payload: {
        pageNo: 1,
        pageSize: 1000,
      }
    });
  }

  handleModalShow = () => {
    const { dispatch } = this.props;
    dispatch({ type: 'tagDetail/showModal' });
  }

  handleModalCancel = () => {
    const { dispatch } = this.props;
    dispatch({ type: 'tagDetail/hideModal' });
  }

  handleModalOk = (data) => {
    const { dispatch } = this.props;
    dispatch({
      type: 'tagDetail/saveSortResult',
      payload: { itemList: data }
    });
  }

  handlePageSubmit = (e) => {
    e.preventDefault()
    const {
      dispatch,
      form: {
        validateFields,
        getFieldsValue,
      },
      tagDetail: {
        operType,
        currentItem,
        filters,
      }
    } = this.props;
    validateFields((errors) => {
      if (errors) return;
      let data = {};
      if (operType === 'create') {
        data = { ...getFieldsValue(), status: Codes.CODE_NORMAL };
      } else if (operType === 'update') {
        const { id, groupId, itemList, name, status } = currentItem;
        data = {
          id,
          groupId,
          name,
          status,
          itemList: (itemList || []).map(item => item.id),
          ...getFieldsValue(),
        };
      }
      dispatch({
        type: `tagDetail/${operType}`,
        payload: data,
        callback: () => {
          dispatch(
            routerRedux.push({
              pathname: '/tag/tagItem',
              search: queryString.stringify(filters),
            })
          );
        }
      })
    });
  }

  handlePageCancel = () => {
    const { dispatch, tagDetail: { filters } } = this.props;
    dispatch({ type: 'tagDetail/initState' });
    dispatch(
      routerRedux.push({
        pathname: '/tag/tagItem',
        search: queryString.stringify(filters),
      })
    );
  }

  render() {
    const { dispatch, form: { getFieldDecorator }, tagDetail, group } = this.props;
    const { itemLoading, currentItem, filters, modalVisible } = tagDetail;
    const { list } = group;
    const { itemList, name, code, type, groupId } = currentItem;

    const itemTableColumns = [{
      title: '位置',
      dataIndex: 'sort',
      key: 'sort',
    },{
      title: '商品名称',
      dataIndex: 'name',
      key: 'name',
    },{
      title: '商品状态',
      dataIndex: 'status',
      key: 'status',
      render: (text, record) => {
        const statusMap = {[Codes.CODE_SALE]: 'success', [Codes.CODE_DELETE]: 'error'};
        return (<Badge status={statusMap[record.status]} text={itemStatuses[record.status]} />);
      },
    },{
      title: '商品类型',
      dataIndex: 'type',
      key: 'type',
      render: (text, record) => (<span>{tagType[record.type]}</span>),
    }];

    const formItemLayout = {
      labelCol: {
        span: 7,
      },
      wrapperCol: {
        span: 12,
      },
    };
    const submitFormLayout = {
      wrapperCol: {
        xs: { span: 24, offset: 0 },
        sm: { span: 10, offset: 7 },
      },
    };

    return (
      <PageHeaderLayout>
        <Spin spinning={itemLoading}>
          <Card>
            <Form layout="horizontal" onSubmit={this.handlePageSubmit}>
              <Form.Item label="标签名称：" hasFeedback {...formItemLayout}>
                {getFieldDecorator('name', {
                  rules: [{ required: true, type: 'string', message: "名称为必填项!" }],
                  initialValue: name,
                })(<Input placeholder="请输入" />)}
              </Form.Item>
              {/* 标签类型已抽象为标签组
              <Form.Item label="标签类型：" hasFeedback {...formItemLayout}>
                {getFieldDecorator('type', {
                  rules: [{ required: true, type: 'string', message: "编号为必填项!" }],
                  initialValue: type || Codes.CODE_COURSE,
                })(
                  <Radio.Group>
                    {Object.keys(tagType).map(key => <Radio value={key} key={key}>{tagType[key]}</Radio>)}
                  </Radio.Group>
                )}
              </Form.Item>
              */}
              <Form.Item label="所属标签组" hasFeedback {...formItemLayout}>
                {getFieldDecorator('groupId', {
                  rules: [{ required: true, type: 'string', message: "标签组为必选项!" }],
                  initialValue: groupId,
                })(
                  <Select
                    showSearch
                    allowClear
                    placeholder="请输入标签组编号或者名称进行筛选"
                    optionFilterProp="children"
                    filterOption={(input, option) => option.props.children.toLowerCase().indexOf(input.toLowerCase()) >= 0}
                  >
                    {list.map(item => <Select.Option value={item.id} key={item.id}>{`${item.merchantName}/${item.name}/${item.code}`}</Select.Option>)}
                  </Select>
                )}
              </Form.Item>
              <Form.Item label="商品排序" {...formItemLayout}>
                <Button onClick={this.handleModalShow} disabled={!(itemList || []).length} type="primary" size="small" icon="edit">排序</Button>
              </Form.Item>
              <Form.Item wrapperCol={{ offset: 7, span: 12 }}>
                <Table
                  locale={{
                    emptyText: <span style={{ color: "#C6D0D6" }}>&nbsp;&nbsp;<Icon type="frown-o"/>
                      该标签下不包含任何商品，无法排序，请先去关联商品吧!</span>
                  }}
                  dataSource={itemList}
                  columns={itemTableColumns}
                  rowKey={record => record.id}
                  bordered
                  pagination={false}
                />
              </Form.Item>
              <Form.Item {...submitFormLayout} style={{ marginTop: 32 }}>
                <Button onClick={this.handlePageCancel}>取消</Button>
                <Button type="primary" style={{ marginLeft: 35 }} htmlType="submit">提交</Button>
              </Form.Item>
            </Form>
            <ItemSortModal
              itemList={itemList || []}
              rowKeyName="id"
              modalVisible={modalVisible}
              onCancel={this.handleModalCancel}
              onOk={this.handleModalOk}
            />
          </Card>
        </Spin>
      </PageHeaderLayout>
    );
  }
}
